clear all;
close all;
clc;

addpath(genpath('lightspeed'));
addpath(genpath('minFunc_2012'));
addpath(genpath('tensor_toolbox_2.6'));
addpath(genpath('util_kernel'));

domain = 'equity';
data_path = strcat('data/', domain, '.mat');
data = load(data_path);

% xtrain, xtest, ytest are normalized to mu=0 std=1, NOT ytest
X_train = data.X_train;
Y_train = data.Y_train;
X_test = data.X_test;

rank = 5;

try
    model = hogp_v2(X_train, tensor(Y_train), X_test, rank);
    % normalized prediction
    N_ystar = model.yPred.data;
    ystar = N_ystar .* data.Y_std + data.Y_mean;
catch
    fprintf('Error occur when training hogp \n');
end

Nrmse = sqrt(mean((data.Y_test - ystar).^2, 'all')) / std(data.Y_test, 0, 'all');
Nmae = mean(abs(data.Y_test - ystar), 'all') / std(data.Y_test, 0, 'all');

rmse = sqrt(mean((data.Y_test - ystar).^2, 'all'));
mae = mean(abs(data.Y_test - ystar), 'all');

fprintf('* HOGP with %s\n', domain);
fprintf('  - nrmse = %.7f, nmae = %.7f\n', Nrmse, Nmae);
fprintf('  - rmse = %.7f, mae = %.7f\n', rmse, mae);